﻿using System;
using System.Collections.Generic;
using System.Linq;
using Microsoft.Xna.Framework;
using Microsoft.Xna.Framework.Audio;
using Microsoft.Xna.Framework.Content;
using Microsoft.Xna.Framework.GamerServices;
using Microsoft.Xna.Framework.Graphics;
using Microsoft.Xna.Framework.Input;
using Microsoft.Xna.Framework.Media;

namespace XNA4ModelAnimation
{
    /// <summary>
    /// This is the main type for your game
    /// </summary>
    public class Game1 : Microsoft.Xna.Framework.Game
    {
        GraphicsDeviceManager graphics;
        SpriteBatch spriteBatch;

        // Set the 3D model to draw.
        ModelNode myModel;

        // Set the position of the model in world space, and set the rotation.
        Vector3 modelPosition = Vector3.Zero;
        Quaternion modelRotation = Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(0));
        // Set the velocity of the model, applied each frame to the model's position.
        Vector3 modelVelocity = Vector3.Zero;
        // Set the position of the camera in world space, for our view matrix.
        Vector3 cameraPosition = new Vector3(0.0f, 3000.0f, 0.0f);
        // The aspect ratio determines how to scale 3d to 2d projection.
        float aspectRatio;
        
        Matrix world;
        Matrix view;
        Matrix projection;

        /// <summary>
        /// input
        /// </summary>
        static InputHandler inputs;
        /// <summary>
        /// FPS Counter
        /// </summary>
        static FpsCounter fpsCounter;
        bool toggleMove = false;
        //float totalElapsedTime;
        //float duration = 5.0f;
        //Vector3 startPosition = Vector3.Zero;
        //Vector3 endPosition = new Vector3(500, 250, 0);

        ModelAnimation anim1;        
        KeyframeModelAnimation anim2;
        CatmullRomCurveModelAnimation anim3;

        enum AnimationMode { LINEAR = 1, KEYFRAME, CURVE };
        AnimationMode g_animmode = AnimationMode.LINEAR;

        public Game1()
        {
            graphics = new GraphicsDeviceManager(this);
            Content.RootDirectory = "Content";
        }

        /// <summary>
        /// Allows the game to perform any initialization it needs to before starting to run.
        /// This is where it can query for any required services and load any non-graphic
        /// related content.  Calling base.Initialize will enumerate through any components
        /// and initialize them as well.
        /// </summary>
        protected override void Initialize()
        {
            // TODO: Add your initialization logic here
            /// <summary>
            /// inputs component
            /// </summary>
            inputs = new InputHandler(this);
            Components.Add(inputs);
            /// <summary>
            /// fps component
            /// </summary>
            fpsCounter = new FpsCounter(this);
            Components.Add(fpsCounter);

            //world = Matrix.CreateRotationY(modelRotationYaw) * Matrix.CreateRotationX(modelRotationPitch) * Matrix.CreateTranslation(modelPosition);
            world = Matrix.CreateFromQuaternion(modelRotation) * Matrix.CreateTranslation(modelPosition);
            view = Matrix.CreateLookAt(cameraPosition, Vector3.Zero, Vector3.Forward);
            aspectRatio = (float)graphics.GraphicsDevice.Viewport.Width / (float)graphics.GraphicsDevice.Viewport.Height;
            projection = Matrix.CreatePerspectiveFieldOfView(MathHelper.ToRadians(45.0f), aspectRatio, 1.0f, 10000.0f);

            myModel = new ModelNode(this, "Model\\p1_wedge", Vector3.Zero, new Vector3(0.5f, 0.5f, 0.5f), Quaternion.Identity);
            myModel.World = world;
            myModel.View = view;
            myModel.Projection = projection;
            Components.Add(myModel);

            InitModelAnimation();
            InitKeyframeModelAnimation();
            InitCurveModelAnimation();

            base.Initialize();
        }

        void InitModelAnimation()
        {
            anim1 = new ModelAnimation(Vector3.Zero, 
                        new Vector3(500, 0, 500),
                        Quaternion.CreateFromAxisAngle(Vector3.Up, 0.0f), 
                        Quaternion.CreateFromAxisAngle(Vector3.Up, -MathHelper.PiOver2), 
                        TimeSpan.FromSeconds(5), true);
        }

        void InitKeyframeModelAnimation()
        {
            List<ModelAnimationFrame> frames = new List<ModelAnimationFrame>();
            // frames
            frames.Add(new ModelAnimationFrame(new Vector3(-500, 0, -500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, -MathHelper.PiOver2),
                TimeSpan.FromSeconds(0)));
            frames.Add(new ModelAnimationFrame(new Vector3(500, 0, -500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-90)),
                TimeSpan.FromSeconds(3)));
            frames.Add(new ModelAnimationFrame(new Vector3(500, 0, -500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-180)),
                TimeSpan.FromSeconds(6)));
            frames.Add(new ModelAnimationFrame(new Vector3(500, 0, 500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-180)),
                TimeSpan.FromSeconds(9)));
            frames.Add(new ModelAnimationFrame(new Vector3(500, 0, 500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-270)),
                TimeSpan.FromSeconds(12)));
            frames.Add(new ModelAnimationFrame(new Vector3(-500, 0, 500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-270)),
                TimeSpan.FromSeconds(15)));
            frames.Add(new ModelAnimationFrame(new Vector3(-500, 0, 500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-360)),
                TimeSpan.FromSeconds(18)));
            frames.Add(new ModelAnimationFrame(new Vector3(-500, 0, -500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-360)),
                TimeSpan.FromSeconds(21)));
            frames.Add(new ModelAnimationFrame(new Vector3(-500, 0, -500),
                Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(-450)),
                TimeSpan.FromSeconds(24)));
            anim2 = new KeyframeModelAnimation(frames, false);
        }

        void InitCurveModelAnimation()
        {
            List<CurveModelAnimationFrame> frames = new List<CurveModelAnimationFrame>();
            // frames
            frames.Add(new CurveModelAnimationFrame(new Vector3(-500, 0, -500),
                TimeSpan.FromSeconds(0)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(500, 0, -500),
                TimeSpan.FromSeconds(3)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(500, 0, 500),
                TimeSpan.FromSeconds(6)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(-500, 0, 500),
                TimeSpan.FromSeconds(9)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(-500, 0, -750),
                TimeSpan.FromSeconds(12)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(750, 0, -750),
                TimeSpan.FromSeconds(15)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(750, 0, 750),
                TimeSpan.FromSeconds(18)));
            frames.Add(new CurveModelAnimationFrame(new Vector3(-750, 0, 750),
                TimeSpan.FromSeconds(21)));
            anim3 = new CatmullRomCurveModelAnimation(frames, false);
        }

        /// <summary>
        /// LoadContent will be called once per game and is the place to load
        /// all of your content.
        /// </summary>
        protected override void LoadContent()
        {
            // Create a new SpriteBatch, which can be used to draw textures.
            spriteBatch = new SpriteBatch(GraphicsDevice);

            // TODO: use this.Content to load your game content here
        }

        /// <summary>
        /// UnloadContent will be called once per game and is the place to unload
        /// all content.
        /// </summary>
        protected override void UnloadContent()
        {
            // TODO: Unload any non ContentManager content here
        }

        /// <summary>
        /// Allows the game to run logic such as updating the world,
        /// checking for collisions, gathering input, and playing audio.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Update(GameTime gameTime)
        {
            // Allows the game to exit
            if (GamePad.GetState(PlayerIndex.One).Buttons.Back == ButtonState.Pressed)
                this.Exit();

            // TODO: Add your update logic here
            // Get some input.
            UpdateInput();

            if (toggleMove)
            {
                /*totalElapsedTime += (float)gameTime.ElapsedGameTime.TotalSeconds;
                float fraction = totalElapsedTime / duration; // determine how far along the duration value we are (0 to 1)
                while (fraction > 1) fraction -= 1; // fraction = MathHelper.Clamp(fraction, 0, 1);
                modelPosition = Vector3.Lerp(startPosition, endPosition, fraction);*/

                if (g_animmode == AnimationMode.LINEAR)
                {
                    anim1.Update(gameTime.ElapsedGameTime);
                    modelPosition = anim1.Position;
                    modelRotation = anim1.Rotation;
                }
                else if (g_animmode == AnimationMode.KEYFRAME)
                {
                    anim2.Update(gameTime.ElapsedGameTime);
                    modelPosition = anim2.Position;
                    modelRotation = anim2.Rotation;
                }
                else if (g_animmode == AnimationMode.CURVE)
                {
                    anim3.Update(gameTime.ElapsedGameTime);
                    modelPosition = anim3.Position;
                    modelRotation = anim3.Rotation;
                }
            }

            // Add velocity to the current position.
            modelPosition += modelVelocity;
            // Bleed off velocity over time.
            modelVelocity *= 0.95f;

            Matrix world = Matrix.CreateFromQuaternion(modelRotation) * Matrix.CreateTranslation(modelPosition);
            myModel.World = world;

            base.Update(gameTime);
        }

        protected void UpdateInput()
        {
            float speed = 50.0f;
            if (inputs.PauseGame)
            {
                this.Exit();
            }
            if (inputs.IsKeyPressed(Keys.Up))
            {
                modelPosition += Vector3.Up * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Down))
            {
                modelPosition += Vector3.Down * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Right))
            {
                modelPosition += Vector3.Right * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Left))
            {
                modelPosition += Vector3.Left * speed;
                System.Diagnostics.Trace.WriteLine("modelPosition=" + modelPosition);
            }
            else if (inputs.IsKeyPressed(Keys.Space))
            {
                modelPosition = Vector3.Zero;
                modelVelocity = Vector3.Zero;
                modelRotation = Quaternion.CreateFromAxisAngle(Vector3.Up, MathHelper.ToRadians(0));
                toggleMove = false;
            }
            else if (inputs.IsKeyPressed(Keys.M))
            {
                toggleMove = !toggleMove;
                System.Diagnostics.Trace.WriteLine("toggleMove=" + toggleMove);
            }

            if (inputs.IsKeyDown(Keys.D1))
            {
                g_animmode = AnimationMode.LINEAR;
                toggleMove = true;
                //System.Diagnostics.Trace.WriteLine("linear interpolation");
            }
            else if (inputs.IsKeyDown(Keys.D2))
            {
                g_animmode = AnimationMode.KEYFRAME;
                toggleMove = true;
            }
            else if (inputs.IsKeyDown(Keys.D3))
            {
                g_animmode = AnimationMode.CURVE;
                toggleMove = true;
            }
        }

        /// <summary>
        /// This is called when the game should draw itself.
        /// </summary>
        /// <param name="gameTime">Provides a snapshot of timing values.</param>
        protected override void Draw(GameTime gameTime)
        {
            GraphicsDevice.Clear(Color.CornflowerBlue);

            // TODO: Add your drawing code here
 
            base.Draw(gameTime);
        }
    }
}
