﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA4ModelAnimation
{
    public class ModelAnimationFrame
    {
        public Vector3 Position { get; private set; }
        public Quaternion Rotation { get; private set; }
        public TimeSpan Time { get; private set; }

        public ModelAnimationFrame(Vector3 Position, Quaternion Rotation, TimeSpan Time)
        {
            this.Position = Position;
            this.Rotation = Rotation;
            this.Time = Time;
        }
    }

    public class KeyframeModelAnimation
    {
        List<ModelAnimationFrame> frames = new List<ModelAnimationFrame>();
        bool loop;
        TimeSpan elapsedTime = TimeSpan.FromSeconds(0);

        public Vector3 Position { get; private set; }
        public Quaternion Rotation { get; private set; }

        public KeyframeModelAnimation(List<ModelAnimationFrame> Frames, bool Loop)
        {
            this.frames = Frames;
            this.loop = Loop;
            Position = Frames[0].Position;
            Rotation = Frames[0].Rotation;
        }

        public void Update(TimeSpan Elapsed)
        {
            // Update the time
            this.elapsedTime += Elapsed;

            TimeSpan totalTime = elapsedTime;
            TimeSpan end = frames[frames.Count - 1].Time;

            if (loop) // Loop around the total time if necessary
            {
                while (totalTime > end)
                    totalTime -= end;
            }
            else
            {
                if (totalTime > end)
                {
                    Position = frames[frames.Count - 1].Position;
                    Rotation = frames[frames.Count - 1].Rotation;
                    return;
                }
            }

            int i = 0;
            // Find the index of the current frame
            while (frames[i + 1].Time < totalTime)
                i++;
            // Find the time since the beginning of this frame
            totalTime -= frames[i].Time;

            // Find how far we are between the current and next frame (0 to 1)
            float fraction = (float)((totalTime.TotalSeconds) /
               (frames[i + 1].Time - frames[i].Time).TotalSeconds);

            // Interpolate position and rotation values between frames
            Position = Vector3.Lerp(frames[i].Position, frames[i + 1].Position, fraction);
            Rotation = Quaternion.Slerp(frames[i].Rotation, frames[i + 1].Rotation, fraction);
        }
    }
}
