﻿using System;
using System.Collections.Generic;
using System.Linq;
using System.Text;
using Microsoft.Xna.Framework;

namespace XNA4ModelAnimation
{
    public class ModelAnimation
    {
        Vector3 startPosition, endPosition;
        Quaternion startQuaternion, endQuaternion;
        TimeSpan duration;
        bool loop;

        public Vector3 Position { get; private set; }
        public Quaternion Rotation { get; private set; }
        public TimeSpan elapsedTime = TimeSpan.FromSeconds(0);

        public ModelAnimation(Vector3 StartPosition, Vector3 EndPosition,
            Quaternion StartQuaternion, Quaternion EndQuaternion, TimeSpan Duration, bool Loop)
        {
            this.startPosition = StartPosition;
            this.endPosition = EndPosition;
            this.startQuaternion = StartQuaternion;
            this.endQuaternion = EndQuaternion;
            this.duration = Duration;
            this.loop = Loop;
            this.Position = startPosition;
            this.Rotation = startQuaternion;
        }

        public void Update(TimeSpan Elapsed)
        {
            // Update the time
            this.elapsedTime += Elapsed;

            // Determine how far along the duration value we are (0 to 1)
            float fraction = (float)elapsedTime.TotalSeconds / (float)duration.TotalSeconds;

            if (loop)
                while (fraction > 1) // Wrap the time if we are looping
                    fraction -= 1;
            else // Clamp to the end value if we are not
                fraction = MathHelper.Clamp(fraction, 0, 1);

            // Update the current position and rotation
            Position = Vector3.Lerp(startPosition, endPosition, fraction);
            Rotation = Quaternion.Slerp(startQuaternion, endQuaternion, fraction);
        }
    }
}
